@@ -21,7 +21,7 @@ from BeautifulSoup import BeautifulSoup, NavigableString
 
 
 # Basic regexp for matching all expressions in a template.
-EXPR = re.compile(r"{([_\w\.]+)}")
+EXPR = re.compile(r"\{([_\w\.]+)\}")
 
 
 class Template(BeautifulSoup):
@@ -73,7 +73,7 @@ def search(tree, data, ns, nstree, remove_empty_nodes, escape):
 
     """
     # Search for all expressions inside a given namespace.
-    regexp = re.compile(r"{%s[_\w\.]*}" % ns.rstrip('.'))
+    regexp = re.compile(r"\{%s[_\w\.]*\}" % ns.rstrip('.'))
     nodes = tree.findAll(replaceable(regexp)) + tree.findAll(text=regexp)
     if not nodes: return tree
 
@@ -86,10 +86,10 @@ def search(tree, data, ns, nstree, remove_empty_nodes, escape):
     # parent node, replacing the appropriate data values.
     # First a little cleanup...
     if not isinstance(data, list): data = [data]
-    data = [v for v in data if v]
+    data = [v for v in data if v not in [None, {}, []]]
     for i, values in enumerate(data):
         # Return a node with the values applied.
-        repl = apply(clone(node), "{%s}" % ns.rstrip('.'),
+        repl = apply(clone(node), "\{%s\}" % ns.rstrip('.'),
                 values, escape)
         # Continue replacement in deeper namespaces.
         for k in nstree:
@@ -124,7 +124,6 @@ def apply(tree, match, data, escape=False):
         data = unicode(data)
     else:
         data = u''
-    if escape: data = cgi.escape(data)
 
     # Escape shrubbery tags inside replacement.
     data = EXPR.sub(lambda m:
@@ -132,13 +131,15 @@ def apply(tree, match, data, escape=False):
             % m.group(1), data)
 
     # Do the replacement.
+    if escape: text = cgi.escape(data)
+    else: text = data
     for node in nodes:
         if isinstance(node, NavigableString):
-            repl = regexp.sub(data, node.string)
+            repl = regexp.sub(text, node.string)
             if node.parent: node.replaceWith(repl)
         else:
             if node.string:
-                node.string = regexp.sub(data, node.string)
+                node.string = regexp.sub(text, node.string)
             for k, v in node.attrs:
                 node[k] = regexp.sub(data, v).strip()
                 if not node[k]: del node[k]
